use core::fmt;
use std::str::FromStr;

use anyhow::Result;

#[derive(Clone, Copy, Debug, Eq, PartialEq)]
pub enum SceneType {
    Rasterizer,
    Raytracer,
}

impl Default for SceneType {
    fn default() -> Self {
        SceneType::Rasterizer
    }
}

impl FromStr for SceneType {
    type Err = anyhow::Error;

    fn from_str(s: &str) -> Result<Self> {
        match s {
            "rasterizer" => Ok(Self::Rasterizer),
            "raytracer" => Ok(Self::Raytracer),

            _ => Err(anyhow::Error::msg(format!(
                "Failed parsing {} for SceneType",
                s
            ))),
        }
    }
}

impl fmt::Display for SceneType {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Rasterizer => write!(f, "rasterizer"),
            Self::Raytracer => write!(f, "raytracer"),
        }
    }
}
